import { assert, assertEquals } from "https://deno.land/std@0.220.1/assert/mod.ts";
import * as path from "https://deno.land/std@0.220.1/path/mod.ts";

export function* range(from: number, to: number) {
	while (from < to) yield from++;
}

export function* indexed<T>(source: Iterable<T>) {
	let index = 0;
	for (const item of source) yield [index++, item] as const;
}

const LOG_LEVEL = 0;
export function log_from(meta: ImportMeta, level = 1) {
	const filename = path.basename(new URL(meta.url).pathname);
	return (...args: unknown[]) => {
		if (LOG_LEVEL < level) return;
		console.log(`[${filename}]`, ...args);
	};
}

export function promise_split<T>() {
	let resolver = null as ((item: T) => void) | null;
	let resolved = false;
	const promise = new Promise<T>((res) => resolver = res);
	const resolve = (item: T) => {
		resolved = true;
		assert(resolver !== null);
		resolver(item);
	};
	return { promise, resolve, resolved };
}

export function sleep(ms: number) {
	return new Promise((resolver) => setTimeout(resolver, ms));
}

export async function clang_format(source: string, ...style_properties: [string, string][]) {
	const encoded_input = new TextEncoder().encode(source);
	const style_arg = `--style={${style_properties.map(([n, v]) => `${n}: ${v}`).join(",")}}`;
	const command = new Deno.Command("clang-format", { args: ["-", style_arg], stdin: "piped", stdout: "piped" });
	const process = command.spawn();
	const writer = process.stdin.getWriter();
	await writer.write(encoded_input);
	await writer.close();
	const result = await process.output();
	if (!result.success) throw new Error(`clang-format failed with style_arg: '${style_arg}'`);
	const encoded_output = result.stdout;
	const output = new TextDecoder().decode(encoded_output);
	return output;
}

Deno.test("try_clang_format", async () => {
	const input = "int a = 3;";
	const output = await clang_format(input);
	assertEquals(input, output);
});
